package zkp;

import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.Serializable;

import zkp.MenuPanelAccessProvider.MenuSignal;

/**
 * A clickable, selectable Button for use in menus
 * 
 * @author zsmb
 */
public class Button extends MouseAdapter implements Serializable {

    private static final long serialVersionUID = -2789740909450729057L;

    /** Height of a Button */
    public static final int height = 44;
    private static final int width = 200; // width of a Button
    
    private static final int activeColour = 0x66AAAAFF; // background colour of
                                                        // an active Button
    private static final int inactiveColour = 0x666666FF; // background colour
                                                          // of an inactive
                                                          // Button
    private static final int activeFontColour = 0x99EEEEFF; // font colour of an
                                                            // active Button
    private static final int inactiveFontColour = 0x999999FF; // font colour of
                                                              // an inactive
                                                              // Button

    private boolean active; // flag showing whether the Button is selected

    private final int x, y; // x and y positions of the Button (upper left
                            // corner)
    private final String label; // the label displayed on the Button

    private MenuPanelAccessProvider mpap; // the Button can signal being activated to
                                          // the MenuPanel containing it via this reference
    private MenuSignal sig; // the Signal the Button should send to the MenuPanel when
                            // activated

    /**
     * Button ctor
     * 
     * @param mpap the interface of the MenuPanel the Button is in
     * @param x the x coordinate of the Button (left edge)
     * @param y the y coordinate of the Button (upper edge)
     * @param label the text to be displayed on the Button
     * @param sig the Signal the Button sends to the MenuPanel it's in when activated
     */
    public Button(MenuPanelAccessProvider mpap, int x, int y, String label, MenuSignal sig) {
        this.mpap = mpap;
        this.x = x;
        this.y = y;
        this.label = label;
        this.sig = sig;
    }

    /**
     * Paints the Button on the screen
     * 
     * @param g the Graphics to use for displaying the Button
     */
    public void paint(Graphics g) {
        GFX.rect(g, x - width / 2, y, width, height, active ? activeColour : inactiveColour);
        GFX.aaText(g, x, y + height * .75, label, active ? activeFontColour : inactiveFontColour, 0, height * .7);
    }
    
    /**
     * Getter function
     * 
     * @return the Button's y position
     */
    public int getY() {
        return y;
    }
    
    /**
     * Setter function for whether the Button is selected
     * 
     * @param active the desired state of the Button
     */
    public void setActive(boolean active) {
        this.active = active;
    }

    /**
     * Activates the Button (sends its signal to the containing MenuPanel)
     */
    public void activate() {
        mpap.signal(sig);
    }

    /**
     * Checks if a mouse click activated the Button
     * 
     * @param mouseX the x coordinate of the mouse click
     * @param mouseY the y coordinate of the mouse click
     * @return true, if the Button was hit by the mouse click
     */
    private boolean hit(int mouseX, int mouseY) {
        return x - width / 2 < mouseX && mouseX < x + width / 2 && y < mouseY && mouseY < y + height;
    }

    /**
     * If the Button was already active and the mouse was released over it, it activates
     * the Button <br>
     * Always deactivates and repaints the Button
     * 
     * @param e the MouseEvent to process
     */
    @Override
    public void mouseReleased(MouseEvent e) {
        if (active && hit(e.getX(), e.getY())) {
            activate();
        }

        active = false;
        mpap.requestRepaint();
    }

    /**
     * Activates and repaints the Button if the mouse down event hit it
     * 
     * @param e the MouseEvent to process
     */
    @Override
    public void mousePressed(MouseEvent e) {
        if (hit(e.getX(), e.getY())) {
            active = true;
            mpap.requestRepaint();
        }
    }
}
