package zkp;

import java.awt.Graphics;
import java.util.Random;

import zkp.GamePanel.ShipClass;

/**
 * Abstract base class for all types of enemies present in the game
 * 
 * @author zsmb
 */
abstract public class EnemyShip extends GameObject {

    private static final long serialVersionUID = -4382924580217395062L;

    private static final int targetedColour = 0xFF00AAFF; // font colour of a
                                                          // non-targeted
                                                          // EnemyShip
    private static final int nonTargetedColour = 0xFFDD00FF; // font colour of a
                                                             // targeted
                                                             // EnemyShip

    protected String word = null; // the word associated with the EnemyShip
    protected int health; // remaining health points
    protected int hitsTaken = 0; // number of projectiles that actually reached
                                 // the EnemyShip

    protected double xVelocity = 0; // velocity in the horizontal direction
                                    // (positive right)
    protected double yVelocity; // velocity in the vertical direction (positive
                                // down)

    protected int radius; // radius of the EnemyShip (both for display and hit
                          // calculation)

    protected boolean isTargeted = false; // flag to store if the EnemyShip is
                                          // currently under attack

    protected GamePanelAccessProvider gpa; // the containing GamePanel's
                                           // interface for communication

    /**
     * EnemyShip ctor
     * 
     * @param gpa the containing GamePanel's interface
     * @param radius the size of the EnemyShip
     */
    public EnemyShip(GamePanelAccessProvider gpa, int radius) {
        this.gpa = gpa;
        this.radius = radius;

        double xPos = getRandomXPosition();
        double yPos = getRandomYPosition();

        super.setXY(xPos, yPos);
    }

    /**
     * EnemyShip ctor with explicit position values
     * 
     * @param gpa the containing GamePanel's interface
     * @param radius the size of the EnemyShip
     * @param x the x coordinate of the EnemyShip
     * @param y the y coordinate of the EnemyShip
     */
    public EnemyShip(GamePanelAccessProvider gpa, int radius, double x, double y) {
        super(x, y);
        this.gpa = gpa;
        this.radius = radius;
    }

    /**
     * Provides a random X position for a spawning EnemyShip
     * 
     * @return the randomized X coordinate
     */
    private double getRandomXPosition() {
        return gpa.getRandomX(this);
    }

    /**
     * Provides a random Y position for a spawning EnemyShip
     * 
     * @return the randomized Y coordinate
     */
    private double getRandomYPosition() {
        Random r = new Random();
        return -10 - r.nextDouble() * 100;
    }
    
    /**
     * Getter function
     * 
     * @return The next character to hit on the EnemyShip
     */
    public char getCurrentChar() {
        return word.charAt(word.length() - health);
    }
    
    /**
     * Getter function
     * 
     * @return the first character of the EnemyShip's word
     */
    public char getKeyChar() {
        return word.charAt(0);
    }

    /**
     * Getter for the remaining part of the EnemyShip's word to be typed
     * 
     * @return the remaining letters to type
     */
    private String getDisplayText() {
        return word.substring(word.length() - health);
    }

    /**
     * Gets the appropriate font colour based on whether the EnemyShip is currently under
     * attack
     * 
     * @return the colour to use for painting the EnemyShip's word
     */
    private int getColour() {
        return isTargeted ? targetedColour : nonTargetedColour;
    }

    /**
     * Getter function
     * 
     * @return the EnemyShip's radius
     */
    public double getRadius() {
        return (double) radius;
    }

    /**
     * Getter function to determine the exact type of the EnemyShip, this is necessary for
     * the kill counting by ship type
     * 
     * @return the ShipClass of the EnemyShip
     */
    public abstract ShipClass getShipClass();

    /**
     * Setter function for the EnemyShip's word
     * 
     * @param word the word to assign to the EnemyShip
     */
    protected void setWord(String word) {
        this.word = word;
        health = word.length();
    }

    private static final int stringPadding = 3; // Some manual offset to
                                                // distance the EnemyShip's word
                                                // from it's circular onscreen
                                                // representation

    /**
     * Paints the word of the EnemyShip on the screen
     * 
     * @param g the Graphics object to use to paint
     * @param xOffset horizontal offset of the displayed word (positive LEFT, as that's
     *            the common direction)
     * @param yOffset vertical offset of the displayed word (positive down)
     */
    protected void paintText(Graphics g, int xOffset, int yOffset) {
        if (isTargeted) {
            GFX.resetFontSize(g); /* This is necessary for the next calculation to work */
            int stringWidth = g.getFontMetrics().stringWidth(getDisplayText());
            GFX.rect(g, x + xOffset - stringPadding - stringWidth, y + yOffset - 13, stringWidth, 16, 0x222222DD);
        }
        GFX.aaText(g, x + xOffset - stringPadding, y + yOffset, getDisplayText(), getColour(), 1);
    }

    /**
     * Moves the EnemyShip ahead by one step in both directions
     * 
     * @return true if the EnemyShip crashed into the PlayerShip
     */
    @Override
    public boolean progress() {
        x += xVelocity;
        y += yVelocity;        
        return gpa.isOffScreen(x, y) || gpa.playerHit(this);
    }

    /**
     * Checks if the EnemyShip has been hit by the number of Projectiles required to
     * destroy it
     * 
     * @return true if the EnemyShip has been hit by the number of Projectiles that it can
     *         take already
     */
    public boolean isDefeated() {
        return hitsTaken == word.length();
    }

    /**
     * Check if the EnemyShip's health has been lowered to 0, note that there could still
     * be Projectiles on the way to visually hit it
     * 
     * @return true if the EnemyShip has no health(letters) remaining
     */
    public boolean isPracticallyDead() {
        return health == 0;
    }

    /**
     * Notifies the EnemyShip of being targeted by the PlayerShip
     */
    public void target() {
        isTargeted = true;
    }

    /**
     * Decreases the health of the EnemyShip by 1 unit
     */
    public void lowerHealth() {
        if (health >= 1) {
            health--;
        }
    }

    /**
     * Increases the hit count of the EnemyShip
     */
    public void hit() {
        if (hitsTaken < word.length()) {
            hitsTaken++;
        }
        y -= 3;
    }

}
