package zkp;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontFormatException;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.io.IOException;

import javax.swing.JLabel;

/**
 * A class for drawing graphics primitives and displaying text in a more convenient way
 * than using Graphics directly
 * 
 * @author zsmb
 */
public abstract class GFX {

    private static final int defaultFontSize = 16;

    /**
     * Sets the Graphics object to draw the next request in the specified colour
     * 
     * @param g the Graphics object to set
     * @param colour specified colour in 0xRRGGBBAA format
     */
    private static void setColour(Graphics g, int colour) {
        g.setColor(new Color((colour >> 24) & 255, (colour >> 16) & 255, (colour >> 8) & 255, (colour) & 255));
    }

    /**
     * Draws a filled rectangle on the screen
     * 
     * @param g the Graphics object to paint with
     * @param x x position of the rectangle (left edge)
     * @param y y position of the rectangle (upper edge)
     * @param w width of the rectangle in pixels
     * @param h height of the rectangle in pixels
     * @param colour colour of the rectangle in 0xRRGGBBAA format
     */
    public static void rect(Graphics g, double x, double y, double w, double h, int colour) {
        setColour(g, colour);
        g.fillRect((int) x, (int) y, (int) w, (int) h);
    }

    /**
     * Draws a filled circle on the screen
     * 
     * @param g the Graphics object to paint with
     * @param x x position of the circle (left edge)
     * @param y y position of the circle (upper edge)
     * @param r radius of the circle, in pixels
     * @param colour colour of the circle in 0xRRGGBBAA format
     */
    public static void circle(Graphics g, double x, double y, double r, int colour) {
        setColour(g, colour);
        g.fillOval((int) (x - r), (int) (y - r), (int) (2 * r), (int) (2 * r));
    }

    /**
     * Draws a line on the screen between the two given points
     * 
     * @param g the Graphics object to paint with
     * @param x1 x position of the first point
     * @param y1 y position of the first point
     * @param x2 x position of the second point
     * @param y2 y position of the second point
     * @param colour colour of the line in 0xRRGGBBAA format
     */
    public static void line(Graphics g, double x1, double y1, double x2, double y2, int colour) {
        setColour(g, colour);
        g.drawLine((int) x1, (int) y1, (int) x2, (int) y2);
    }

    /**
     * Displays font on the screen using the default font (left-aligned)
     * 
     * @param g the Graphics object to paint with
     * @param x x position of the text (left edge)
     * @param y y position of the text (lower edge)
     * @param text the text to display
     * @param colour colour of the text in 0xRRGGBBAA format
     */
    public static void text(Graphics g, double x, double y, String text, int colour) {
        text(g, x, y, text, colour, -1);
    }

    /**
     * Displays font on the screen using the default font
     * 
     * @param g the Graphics object to paint with
     * @param x x position of the text
     * @param y y position of the text (lower edge)
     * @param text the text to display
     * @param colour colour of the text in 0xRRGGBBAA format
     * @param align -1 for left, 0 for centered, 1 for right aligned
     */
    public static void text(Graphics g, double x, double y, String text, int colour, int align) {

        g.setFont(new JLabel().getFont());
        setColour(g, colour);

        int width = g.getFontMetrics().stringWidth(text);

        switch (align) {
        case -1: /* left align */
            g.drawString(text, (int) x, (int) y);
            break;
        case 0: /* center align */
            g.drawString(text, (int) (x - width / 2), (int) y);
            break;
        case 1: /* right align */
            g.drawString(text, (int) (x - width), (int) y);
            break;

        default:
            System.err.println("Invalid align parameter");
        }
    }

    /**
     * The antialiased font used in the functions below
     */
    private static Font aaFont = null;

    /**
     * Renders antialiased text on the screen
     * 
     * @param g the Graphics object to paint with
     * @param x x position of the text
     * @param y y position of the text (lower edge)
     * @param text the text to display
     * @param colour colour of the text in 0xRRGGBBAA format
     * @param align -1 for left, 0 for centered, 1 for right aligned
     * @param fontSize size of the text in pixels
     */
    public static void aaText(Graphics g, double x, double y, String text, int colour, int align, double fontSize) {

        if (aaFont == null) {
            try {
                aaFont = Font.createFont(Font.TRUETYPE_FONT, GFX.class.getResourceAsStream("/resources/SourceSansPro-Regular.ttf"));
                        //new File("/resources/SourceSansPro-Regular.ttf"));
            } catch (FontFormatException | IOException e) {
                e.printStackTrace();
            }
        }

        aaFont = aaFont.deriveFont(Font.TRUETYPE_FONT, (int) fontSize);
        Graphics2D g2d = (Graphics2D) g;
        g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

        g.setFont(aaFont);
        setColour(g, colour);

        int width = g.getFontMetrics().stringWidth(text);

        switch (align) {
        case -1: /* left align */
            g.drawString(text, (int) x, (int) y);
            break;
        case 0: /* center align */
            g.drawString(text, (int) (x - width / 2), (int) y);
            break;
        case 1: /* right align */
            g.drawString(text, (int) (x - width), (int) y);
            break;

        default:
            System.err.println("Invalid align parameter");
        }
    }

    /**
     * Renders antialiased text on the screen
     * 
     * @param g the Graphics object to paint with
     * @param x x position of the text
     * @param y y position of the text (lower edge)
     * @param text the text to display
     * @param colour colour of the text in 0xRRGGBBAA format
     * @param align -1 for left, 0 for centered, 1 for right aligned
     */
    public static void aaText(Graphics g, double x, double y, String text, int colour, int align) {
        aaText(g, x, y, text, colour, align, 16);
    }

    /**
     * Renders antialiased text on the screen (left aligned)
     * 
     * @param g the Graphics object to paint with
     * @param x x position of the text
     * @param y y position of the text (lower edge)
     * @param text the text to display
     * @param colour colour of the text in 0xRRGGBBAA format
     */
    public static void aaText(Graphics g, double x, double y, String text, int colour) {
        aaText(g, x, y, text, colour, -1, 16);
    }

    /**
     * Resets the Graphics object's font to the standard sized antialiased font
     * 
     * @param g the Graphics object to reset
     */
    public static void resetFontSize(Graphics g) {
        aaFont = aaFont.deriveFont(Font.TRUETYPE_FONT, defaultFontSize);
        g.setFont(aaFont);
    }

}
