package zkp;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import javax.swing.JFrame;

/**
 * The JFrame that the whole game is contained in, responsible for displaying the current
 * panel and navigation between them
 *
 * @author zsmb
 */
public class GameFrame extends JFrame implements GameFrameAccessProvider {

    private static final long serialVersionUID = 4745659520489342200L;

    private final int width, height; // width and height of the frame in pixels

    private PanelViewManager pvm; // aids switching between which screen to
                                  // display
    private StatCollector sc; // collects and provides access to game statistics

    private int currentLevel = 1; // the number of the current level, indexing
                                  // starts at 1

    /**
     * GameFrame ctor
     * 
     * @param width width in pixels
     * @param height height in pixels
     */
    public GameFrame(int width, int height) {
        this.width = width;
        this.height = height;

        setSize(width, height);
        setResizable(false);
        setTitle("ZedKeyPress");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        pvm = new PanelViewManager(this);

        if (serialStatCollector.exists()) {
            loadStats();
        }
        else {
            sc = new StatCollector();
        }

        if (serialGamePanel.exists()) {
            loadGamePanel();
        }

        pvm.add(new MenuPanel(this, width, height), 1);
        pvm.add(new StatPanel(this, width, height, sc), 2);

        this.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                saveStats();
                if (pvm.exists(0)) {
                    saveGamePanel();
                }
            }
        });

        pvm.show(1);
    }

    /**
     * Starts a new level
     */
    private void play() {
        GamePanel gp = new GamePanel(this, sc, width, height, currentLevel);
        pvm.add(gp, 0);
        pvm.show(0);
        gp.run();
    }

    /** The file to save a paused GamePanel to */
    private File serialGamePanel = new File("sergp.dat");

    /**
     * Writes a paused GamePanel to file, as well as the number of the current level
     */
    private void saveGamePanel() {
        try {
            FileOutputStream fout = new FileOutputStream(serialGamePanel);
            ObjectOutputStream oout = new ObjectOutputStream(fout);
            oout.writeObject(pvm.get(0));
            oout.writeObject(currentLevel);
            oout.close();
        } catch (Exception e) {
            //e.printStackTrace();
            serialGamePanel.delete();
        }
    }

    /**
     * Loads a paused GamePanel from file, as well restores the current level number
     */
    private void loadGamePanel() {
        try {
            FileInputStream fin = new FileInputStream(serialGamePanel);
            ObjectInputStream oin = new ObjectInputStream(fin);

            GamePanel gp = (GamePanel) oin.readObject();
            gp.reInitialize(this, sc);
            currentLevel = (int) oin.readObject();
            oin.close();
            pvm.add(gp, 0);
        } catch (Exception e) {
            //e.printStackTrace();
            serialGamePanel.delete();
        }
    }

    /** The file to save statistics in */
    private File serialStatCollector = new File("sersc.dat");

    /**
     * Writes the StatCollector to file
     */
    private void saveStats() {
        try {
            FileOutputStream fout = new FileOutputStream(serialStatCollector);
            ObjectOutputStream oout = new ObjectOutputStream(fout);
            oout.writeObject(sc);
            oout.close();
        } catch (Exception e) {
            //e.printStackTrace();
            serialStatCollector.delete();
        }
    }

    /**
     * Loads the StatCollector from file
     */
    private void loadStats() {
        try {
            FileInputStream fin = new FileInputStream(serialStatCollector);
            ObjectInputStream oin = new ObjectInputStream(fin);
            sc = (StatCollector) oin.readObject();
            oin.close();
        } catch (Exception e) {
            //e.printStackTrace();
            serialStatCollector.delete();
        }
    }

    /**
     * Processes control signals received from contained Panels
     * 
     * @param fs the received signal
     */
    @Override
    public void signal(FrameSignal fs) {
        switch (fs) {

        case Play:
            currentLevel = 1;
            play();
            break;

        case Exit:
            this.dispatchEvent(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
            break;

        case Menu:
            pvm.show(1);
            break;

        case Stats:
            pvm.show(2);
            break;

        case Resume:
            if (pvm.exists(0)) {
                GamePanel gp = (GamePanel) pvm.get(0);
                gp.resume();
                pvm.show(0);
            }
            break;

        case LevelComplete:
            currentLevel++;
            play();
            break;

        case LevelFailed:
            sc.commitScore();
            currentLevel = 1;

            StatPanel sp = (StatPanel) pvm.get(2);
            sp.notifyGameEnded();

            pvm.show(2);
            pvm.remove(0);
            break;
        }
    }

}
