package zkp;

import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.swing.Timer;

import zkp.GameFrameAccessProvider.FrameSignal;

/**
 * The panel that contains a whole level of a game, it coordinates everything from
 * initialization to gameplay to the initial end of the level, when it signals the
 * GameFrame that contains it
 *
 * @author zsmb
 */
public class GamePanel extends RepaintablePanel {

    private static final long serialVersionUID = -6795246664274691070L;

    private List<Projectile> projectiles = new ArrayList<Projectile>(); // all Projectiles
                                                                        // are stored here
    private SortedMap<Character, List<EnemyShip>> enemies 
        = new TreeMap<Character, List<EnemyShip>>(); // EnemyShips are stored here, sorted
                                                     // into lists by their key characters
    
    private Set<EnemyShip> killQueue = new HashSet<EnemyShip>(); // EnemyShips to be
                                                                 // killed when possible
    
    private Set<EnemyShip> spawnQueue = new HashSet<EnemyShip>(); // EnemyShips to spawn
                                                                  // when possible

    private EnemyShip currentEnemy; // the currently targeted EnemyShip
    private PlayerShip player; // the player's ship

    private GameFrameAccessProvider gfa; // the containing GameFrame's interface
                                         // for communication
    private StatCollector sc; // a StatCollector to send ingame statistics to

    private final int width, height; // width and height in pixels

    private final int timerDelay = 17; // milliseconds between two timer events

    private int tickCount = 0; // count of total timer events
    private int wordCount = 0; // count of words successfully typed

    private KeyPressHandler kph = new KeyPressHandler(); // processes keyboard
                                                         // input
    private TimerHandler th = new TimerHandler(); // processes timer events

    private Timer tm = new Timer(timerDelay, th); // timer to keep things moving

    public EnemyShipRequestHandler gpa 
        = new EnemyShipRequestHandler(); // an interface to provide to contained
                                         // EnemyShips
    
    private final Integer levelNumber; // the number of the current level

    /**
     * The various types of ships present in the game
     *
     * @author zsmb
     */
    public enum ShipClass {
        Simple, Explorer, Shuttle, Mothership, Fighter
    }

    /**
     * GamePanel ctor
     * 
     * @param gfa the containing GameFrame's interface for communication
     * @param sc the StatCollector to send statistics to
     * @param width width in pixels
     * @param height height in pixels
     * @param levelNumber number of the currently playing level
     */
    public GamePanel(GameFrameAccessProvider gfa, StatCollector sc, int width, int height, int levelNumber) {

        this.gfa = gfa;
        this.sc = sc;
        this.width = width;
        this.height = height;
        this.levelNumber = levelNumber;

        for (char c = 'a'; c <= 'z'; c++) {
            enemies.put(c, new ArrayList<EnemyShip>());
        }

        player = new PlayerShip(width / 2, height - 70);

        spawnShips();

        addKeyListener(kph);
    }

    /**
     * Adds an EnemyShip to the correct list based on its word
     * 
     * @param es - the EnemyShip to be added
     */
    private void addEnemy(EnemyShip es) {
        enemies.get(es.getKeyChar()).add(es);
    }

    /**
     * Spawns an EnemyShip of the specified type
     * 
     * @param sc the type of EnemyShip to spawn
     */
    private void addEnemy(ShipClass sc) {
        EnemyShip es;
        switch (sc) {
        case Explorer:
            es = new ExplorerShip(gpa);
            break;
        case Mothership:
            es = new MotherShip(gpa);
            break;
        case Simple:
            es = new SimpleShip(gpa);
            break;
        default:
            es = null;
            break;
        }

        if (es != null) {
            addEnemy(es);
        }

    }

    /**
     * Spawns ships on the GamePanel according to the difficulty of the current level
     * being played
     * 
     * @param levelNumber the current level's number
     */
    private void spawnShips() {

        double multiplier = 2;

        int numOfSimpleShips = 4 + (int) (levelNumber * multiplier / 05);
        int numOfExplorerShips = 0 + (int) (levelNumber * multiplier / 10);
        int numOfMotherShips = 0 + (int) (levelNumber * multiplier / 15);

        for (int i = 0; i < numOfSimpleShips; i++)
            addEnemy(ShipClass.Simple);
        for (int i = 0; i < numOfExplorerShips; i++)
            addEnemy(ShipClass.Explorer);
        for (int i = 0; i < numOfMotherShips; i++)
            addEnemy(ShipClass.Mothership);
    }

    /**
     * Starts the generation of timer events
     */
    public void run() {
        tm.start();
    }

    /**
     * Reinitializes the GamePanel after being loaded from a file
     * 
     * @param gfa the current GameFrame's interface
     * @param sc the current GameFrame's StatCollector
     */
    public void reInitialize(GameFrameAccessProvider gfa, StatCollector sc) {
        this.gfa = gfa;
        this.sc = sc;
        gpa.reInit();
    }

    /**
     * Restarts timer, gets focus for listeners
     */
    public void resume() {
        tm.restart();
        this.requestFocusInWindow();
    }

    /**
     * Gets a descriptive name for the current level
     * 
     * @return the name of the current level
     */
    private String getLevelString() {
        return "Level " + levelNumber;
    }

    /**
     * Displays information about the current level in the background
     * 
     * @param g the Graphics object to paint with
     */
    private void displayLevelNumber(Graphics g) {
        
        /* Normal display mode */
        GFX.aaText(g, width / 2, 70, getLevelString(), 0x2A2A2AFF, 0, 48);

        /* Fade in and out alternative display mode */
        
        // if(tickCount > 3*0x55) {
        // /* Do nothing above 255 */
        // }
        // else if(tickCount < 1*0x55) {
        // GFX.aaText(g, width/2, 70, getLevelString(), 0x44444400 +
        // tickCount*3, 0, 36);
        // }
        // else if(tickCount < 2*0x55) {
        // GFX.aaText(g, width/2, 70, getLevelString(), 0x444444FF, 0, 36);
        // }
        // else if(tickCount < 3*0x55) {
        // GFX.aaText(g, width/2, 70, getLevelString(), 0x444444FF -
        // (tickCount-2*0x55)*3, 0, 36);
        // }
    }

    /**
     * Draws all game objects on the screen
     * 
     * @param g the Graphics object to paint with
     */
    @Override
    public void paint(Graphics g) {
        // Background
        GFX.rect(g, 0, 0, this.getWidth(), this.getHeight(), 0x333333FF);
        
        // Grid
//        int iOffset = tickCount%(height/4);
//        int jOffset = tickCount%30;
//        for(int i = -height/2; i < width; i += 30) {
//            GFX.line(g, i+iOffset, 0, i+height/4+iOffset, height, 0x55555544);
//        }
//        for(int j = -100; j < height+height/4; j += 30) {
//            GFX.line(g, 0, j+jOffset, width, (j-width/4)+jOffset, 0x55555544);
//        }
        
        displayLevelNumber(g);

        player.paint(g);
        
        for (List<EnemyShip> el : enemies.values()) {
            for (EnemyShip e : el) {
                e.paint(g);
            }
        }
        if (currentEnemy != null) {
            currentEnemy.paint(g);
        }
        for (Projectile p : projectiles) {
            p.paint(g);
        }
    }

    /**
     * Contains the two signals that the timer processor can emit to signal that the level
     * has ended in one way or another
     *
     * @author zsmb
     */
    private enum LevelEndSignal {
        PlayerDied, NoEnemiesLeft
    }

    /**
     * Stops the timer and notifies the containing GameFrame
     * 
     * @param sig a signal specifying the way the level has ended
     */
    private void levelOver(LevelEndSignal sig) {
        sc.addWordPerMinStat(wordCount, (int) (tickCount / (1000.0 / timerDelay)));
        tm.stop();

        switch (sig) {
        case PlayerDied:
            try {
                Thread.sleep(1500);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            gfa.signal(FrameSignal.LevelFailed);
            break;
        case NoEnemiesLeft:
            gfa.signal(FrameSignal.LevelComplete);
            break;
        }
    }

    /**
     * Checks if all enemies have been killed
     * 
     * @return true if there are no EnemyShips left
     */
    private boolean noEnemiesLeft() {
//        boolean enemiesLeft = false; // assume victory
//        for (List<EnemyShip> list : enemies.values()) {
//            if (list.size() > 0) {
//                enemiesLeft = true;
//            }
//        }
//        return !enemiesLeft;
        return getAllEnemies().size() == 0;
    }

    /**
     * Returns a merged set of all live EnemyShips
     * 
     * @return the set of EnemyShip
     */
    private Set<EnemyShip> getAllEnemies() {
        HashSet<EnemyShip> es = new HashSet<EnemyShip>();

        for (List<EnemyShip> el : enemies.values()) {
            es.addAll(el);
        }

//        es.addAll(killQueue);
//        es.addAll(spawnQueue);

        return es;
    }

    /**
     * This inner class handles the events generated by the timer, it contains the main
     * event loop responsible for the running of the game
     *
     * @author zsmb
     */
    private class TimerHandler implements Serializable, ActionListener {

        private static final long serialVersionUID = 8757485136336831526L;

        /**
         * Adds an EnemyShip to the list of ships to spawn
         * 
         * @param es the EnemyShip to spawn when possible
         */
        private void addToQueue(EnemyShip es) {
            spawnQueue.add(es);
        }

        /**
         * Processes a timer event (moves and repaints all objects)
         * 
         * @param e the Timer event to process
         */
        @Override
        public void actionPerformed(ActionEvent e) {
            tickCount++;
            
            // Check if the level is completed
            if (noEnemiesLeft()) {
                levelOver(LevelEndSignal.NoEnemiesLeft);
                return;
            }

            // Notify PlayerShip of timer event just in case
            player.progress();

            /** Collects Projectiles that hit ships during this tick */
            List<Projectile> projectilesToDelete = new ArrayList<Projectile>();
            
            // For all Projectiles, check if it hit its target
            for (Projectile p : projectiles) {
                if (p.progress()) {
                    EnemyShip target = p.getTarget();
                    target.hit();
                    
                    if (target.isDefeated()) {
                        // Mark EnemyShip as killed
                        killQueue.add(target);
                        
                        // Log to statistics
                        wordCount++;
                        sc.addDefeatedShip(target.getShipClass());
                    }
                    
                    // Mark Projectile as used up
                    projectilesToDelete.add(p);
                }
            }
            
            // Remove all collected Projectiles
            for (Projectile p : projectilesToDelete) {
                projectiles.remove(p);
            }

            // Move EnemyShips forward
            for (EnemyShip es : getAllEnemies()) {
                // Check if they're offscreen
                if (es.progress()) {
                    killQueue.add(es);
                }
            }

            // Delete killed EnemyShips
            for (EnemyShip es : killQueue) {
                List<EnemyShip> el;
                el = enemies.get(es.getKeyChar());
                el.remove(es);
            }
            killQueue.clear();

            // Fulfill EnemyShip spawn requests
            for (EnemyShip es : spawnQueue) {
                addEnemy(es);
            }
            spawnQueue.clear();

            // Update screen
            repaint();
        }
    }

    /**
     * This inner class handles keyboard input ingame
     * 
     * @author zsmb
     */
    private class KeyPressHandler extends KeyAdapter implements Serializable {

        private static final long serialVersionUID = -1623047295695818973L;
        
        /**
         * Looks for a target EnemyShip based on the key pressed and targets it
         * if it is found
         * @param keyCode - the key pressed
         */
        private void searchForTarget(char keyCode) {
            List<EnemyShip> enemyList = enemies.get(keyCode);

            EnemyShip candidate = null;
            for (EnemyShip es : enemyList) {
                // Requirements to be a valid option
                if (!es.isPracticallyDead() && !gpa.isNotYetOnScreen(es.getX(), es.getY())) {
                    // Requirements to be better than current candidate
                    if (candidate == null || gpa.getDistanceLeft(candidate) > gpa.getDistanceLeft(es)) {
                        candidate = es;
                    }
                }
            }

            if (candidate != null) {
                currentEnemy = candidate;
                currentEnemy.target();
            }
        }
        
        /**
         * Attemps to fire at the current enemy with the given key
         * @param keyCode - the key pressed
         */
        private void attemptFire(char keyCode) {
            if (currentEnemy != null && currentEnemy.getCurrentChar() == keyCode) {
                Projectile p = new Projectile(player.x, player.y, currentEnemy);
                projectiles.add(p);
                currentEnemy.lowerHealth();
                if (currentEnemy.isPracticallyDead()) {
                    currentEnemy = null;
                }
            }
        }
        
        /**
         * Handles keypresses
         * 
         * @param ke the key pressed
         */
        @Override
        public void keyPressed(KeyEvent ke) {

            if (ke.getKeyCode() == KeyEvent.VK_ESCAPE) {
                tm.stop();
                gfa.signal(FrameSignal.Menu);
                return;
            }

            char keyCode = Character.toLowerCase(ke.getKeyChar());

            if ('a' <= keyCode && keyCode <= 'z') {
                // If there's no active target, get one
                if (currentEnemy == null) {
                    searchForTarget(keyCode);
                }
                
                // Attempt to fire at currentEnemy
                attemptFire(keyCode);
            }
        }

    }

    /**
     * @author zsmb Inner class that provides a communicatoin interface for EnemyShips
     *         contained by the GamePanel
     */
    private class EnemyShipRequestHandler implements Serializable, GamePanelAccessProvider {

        private static final long serialVersionUID = 6038509703817063590L;

        /**
         * Used for assigning random words to EnemyShips
         */
        private transient RandomWordProvider wordProvider = new RandomWordProvider();
        
        protected void reInit() {
            wordProvider = new RandomWordProvider();
        }
        
        /**
         * Getter function
         * 
         * @return the GamePanel's width
         */
        public int getWidth() {
            return width;
        }

        /**
         * Getter function
         * 
         * @return the GamePanel's height
         */
        public int getHeight() {
            return height;
        }

        /**
         * Request a FighterShip to be spawned
         * 
         * @param spawned the FighterShip to spawn
         * @param spawner the MotherShip spawning the FighterShip
         */
        public void requestSpawn(FighterShip spawned, MotherShip spawner) {
            if (getAllEnemies().contains(spawner)) {
                th.addToQueue(spawned);
            }
        }

        /**
         * Request a ShuttleShip to be spawned
         * 
         * @param spawned the ShuttleShip to spawn
         * @param spawner the ExplorerShip spawning the ShuttleShip
         */
        public void requestSpawn(ShuttleShip spawned, ExplorerShip spawner) {
            if (getAllEnemies().contains(spawner)) {
                th.addToQueue(spawned);
            }
        }

        /**
         * Get a random word for a spawning ship
         * 
         * @param sc the type of EnemyShip to get a word for
         * @return a random word
         */
        @Override
        public String getWord(ShipClass sc) {
            return wordProvider.nextWord(sc, getAllEnemies());
        }

        /**
         * Checks if a given position is offscreen
         * 
         * @param x x coordinate to check
         * @param y y coordinate to check
         */
        @Override
        public boolean isOffScreen(double x, double y) {
            return x < 10 || x > width + 30 || y > height + 25;
        }
        
        /**
         * Checks if a given position is above the screen
         * 
         * @param x x coordinate to check
         * @param y y coordinate to check
         * @return
         */
        @Override
        public boolean isNotYetOnScreen(double x, double y) {
            return y < 0;
        }

        /**
         * Get a random x position for a spawning ship
         * 
         * @param es the EnemyShip to get an x position for
         * @return the random x position
         */
        @Override
        public int getRandomX(EnemyShip es) {
            int rad = (int) es.getRadius();
            int min = 5 + rad * 7;
            int max = width - 10 - rad;

            return Util.getRandomNumber(min, max);
        }

        /**
         * Gets the unit speed for the given GamePanel<br>
         * Default unit speed makes a unit get to the PlayerShip in about 20 seconds
         * 
         * @return the unit speed
         */
        @Override
        public double getUnitSpeed() {
            double numOfTicksInASecond = 1000.0 / timerDelay;
            return player.getY() / (numOfTicksInASecond * 20.0);
        }

        /**
         * Gets an x direction velocity that will make the EnemyShip directly crash into
         * the PlayerShip
         * 
         * @param es the EnemyShip to get a velocity for
         * @return the collision course x velocity
         */
        @Override
        public double getHorizontalCollisionCourseSpeed(EnemyShip es) {
            double xOffset = width / 2 - es.getX();
            double yOffset = player.getY() - es.getY();
            double ySpeed = getUnitSpeed();

            double xSpeed = xOffset * (ySpeed / yOffset);

            return xSpeed;
        }

        /**
         * Get the vertical distance left between an EnemyShip and the PlayerShip
         * 
         * @param es the EnemyShip to calculate the distance for
         * @return the vertical distance left
         */
        @Override
        public double getDistanceLeft(EnemyShip es) {
            return player.getY() - es.getY();
        }

        /**
         * Signals the GamePanel that an EnemyShip is to be removed
         * 
         * @param es the EnemyShip to remove
         */
        @Override
        public void registerToBeRemoved(EnemyShip es) {
            killQueue.add(es);
        }

        /**
         * Check if an EnemyShip crashed into the PlayerShip
         * 
         * @param es the EnemyShip to check
         * @return true if the crash occured
         */
        @Override
        public boolean playerHit(EnemyShip es) {
            double xDist = player.getX() - es.getX();
            double yDist = player.getY() - es.getY();

            double dist = Math.sqrt(xDist * xDist + yDist * yDist);

            boolean playerIsHit = dist < es.getRadius() + player.getRadius() - 3;

            if (playerIsHit) {
                levelOver(LevelEndSignal.PlayerDied);
            }

            return playerIsHit;
        }
    }

    /**
     * Force a repaint of the GamePanel
     */
    @Override
    public void requestRepaint() {
        repaint();
    }

    /**
     * Keeps listeners working while switching panels
     */
    public void addNotify() {
        super.addNotify();
        requestFocus();
    }

}
