package zkp;

import java.io.Serializable;

import zkp.GamePanel.ShipClass;

/**
 * An interface that EnemyShips can use to communicate with their containing GamePanel
 * 
 * @author zsmb
 */
public interface GamePanelAccessProvider extends Serializable {

    /**
     * GamePanel width getter function
     * 
     * @return the GamePanel's width
     */
    public int getWidth();

    /**
     * GamePanel height getter function
     * 
     * @return the GamePanel's height
     */
    public int getHeight();

    /**
     * Get a random x position for a spawning ship
     * 
     * @param es the EnemyShip to get an x position for
     * @return the random x position
     */
    public int getRandomX(EnemyShip es);

    /**
     * Get a random word for a spawning ship
     * 
     * @param sc the type of EnemyShip to get a word for
     * @return a random word
     */
    public String getWord(ShipClass sc);

    /**
     * Gets the unit speed for the given GamePanel
     * 
     * @return the unit speed
     */
    public double getUnitSpeed();

    /**
     * Gets an x direction velocity that will make the EnemyShip directly crash into the
     * PlayerShip
     * 
     * @param es the EnemyShip to get a velocity for
     * @return the collision course x velocity
     */
    public double getHorizontalCollisionCourseSpeed(EnemyShip es);

    /**
     * Get the vertical distance left between an EnemyShip and the PlayerShip
     * 
     * @param es the EnemyShip to calculate the distance for
     * @return the vertical distance left
     */
    public double getDistanceLeft(EnemyShip es);

    /**
     * Checks if a given position is outside the screen boundaries
     * 
     * @param x x coordinate to check
     * @param y y coordinate to check
     * @return true if coordinates are offscreen
     */
    public boolean isOffScreen(double x, double y);
    
    /**
     * Checks if a given position is above the screen
     * 
     * @param x x coordinate to check
     * @param y y coordinate to check
     * @return true if coordinates are above the screen
     */
    public boolean isNotYetOnScreen(double x, double y);

    /**
     * Check if an EnemyShip crashed into the PlayerShip
     * 
     * @param es the EnemyShip to check
     * @return true if the crash occured
     */
    public boolean playerHit(EnemyShip es);

    /**
     * Signals the GamePanel that an EnemyShip is to be removed
     * 
     * @param es the EnemyShip to remove
     */
    public void registerToBeRemoved(EnemyShip es);

    /**
     * Request a FighterShip to be spawned
     * 
     * @param spawned the FighterShip to spawn
     * @param spawner the MotherShip spawning the FighterShip
     */
    public void requestSpawn(FighterShip spawned, MotherShip spawner);

    /**
     * Request a ShuttleShip to be spawned
     * 
     * @param spawned the ShuttleShip to spawn
     * @param spawner the ExplorerShip spawning the ShuttleShip
     */
    public void requestSpawn(ShuttleShip spawned, ExplorerShip spawner);

}
