package zkp;

import java.awt.Graphics;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import zkp.GameFrameAccessProvider.FrameSignal;

/**
 * This panel is the main menu of the game, and it offers the Resume, Play, Stats, and
 * Exit options. Can be navigated with either mouse clicks or the arrow and return keys.
 *
 * @author zsmb
 */
public class MenuPanel extends RepaintablePanel implements MenuPanelAccessProvider {

    private static final long serialVersionUID = -6477299931578895475L;

    private GameFrameAccessProvider gfa; // the containing GameFrame's interface for
                                         // communication
    private final int width, height; // width and height of the panel, in pixels

    private int selectedItemIndex = -1; // the index of the currently selected Button
    private ArrayList<Button> buttons = new ArrayList<Button>(); // the Buttons contained
                                                                 // in the MenuPanel

    /**
     * MenuPanel ctor
     * 
     * @param gfa the containing GameFrame's interface for communication
     * @param width the MenuPanel's width in pixels
     * @param height the MenuPanel's height in pixels
     */
    public MenuPanel(GameFrameAccessProvider gfa, int width, int height) {
        this.gfa = gfa;
        this.width = width;
        this.height = height;

        this.addKeyListener(new KeyPressHandler());
        this.addMouseListener(new ClickDetector());

        /* This unfortunately needs hardcoding */
        int numOfButtons = 4;
        int totalMenuHeight = (int) ((numOfButtons - 1) * (Button.height * 1.25)) + Button.height;
        /* Padding of 25 needed here for title bar */
        currentButtonHeight = (height - 25) / 2 - totalMenuHeight / 2;
        /* Make it slightly higher than centered */
        currentButtonHeight *= (2.0 / 3);

        addButton("Resume", MenuSignal.Resume);
        addButton("Play", MenuSignal.Play);
        addButton("Stats", MenuSignal.Stats);
        addButton("Exit", MenuSignal.Exit);

        this.requestFocusInWindow();
    }

    /**
     * Paints the MenuPanel on the screen
     * 
     * @param g the Graphics object to paint with
     */
    @Override
    public void paint(Graphics g) {
        // Background
        GFX.rect(g, 0, 0, width, height, 0xAAAAAAFF);
        
        for(int i = 0; i < 6; i++) {
            GFX.aaText(g, width/2+i, buttons.get(0).getY()-28+i, "Zed KeyPress",
                    0x99EEEEFF-6*0x19191900+i*0x19191900, 0, 54);
        }

        if (isInButtonIndexRange(selectedItemIndex)) {
            buttons.get(selectedItemIndex).setActive(true);
        }

        for (Button b : buttons) {
            b.paint(g);
        }
    }

    /**
     * Incremented every time a Button is added, always contains height where the next
     * Button should be added
     */
    private int currentButtonHeight;

    /**
     * To be used for checking if a given index can be used on the list of Buttons without
     * exceptions
     * 
     * @param index the index to check
     * @return true if the index is in the required range
     */
    private boolean isInButtonIndexRange(int index) {
        return 0 <= index && index < buttons.size();
    }

    /**
     * Adds a Button to the MenuPanel
     * 
     * @param label the text to display on the Button
     * @param sig the Signal the Button should send upon being activated
     */
    private void addButton(String label, MenuSignal sig) {
        Button b = new Button(this, width / 2, currentButtonHeight, label, sig);
        buttons.add(b);
        currentButtonHeight += Button.height * 1.25;
        // Register the Button to be able to listen to mouse click events
        this.addMouseListener(b);
        this.setVisible(true);
    }

    /**
     * Required for KeyListeners to work
     */
    @Override
    public void addNotify() {
        super.addNotify();
        requestFocus();
    }

    /**
     * Unselects all Buttons
     */
    private void setAllButtonsInactive() {
        for (Button b : buttons) {
            b.setActive(false);
        }
    }

    /**
     * Selects the Button above the currently selected Button, or the last Button, if none
     * were selected
     */
    private void moveSelectionUp() {
        setAllButtonsInactive();
        selectedItemIndex--;
        if (selectedItemIndex < 0) {
            selectedItemIndex = buttons.size() - 1;
        }
    }

    /**
     * Selects the Button below the currently selected Button, or the first Button, if
     * none were selected
     */
    private void moveSelectionDown() {
        setAllButtonsInactive();
        selectedItemIndex++;
        if (selectedItemIndex >= buttons.size()) {
            selectedItemIndex = 0;
        }
    }

    /**
     * Inner class for processing keyboard input
     *
     * @author zsmb
     */
    private class KeyPressHandler extends KeyAdapter {

        /**
         * Moves selection if the UP or DOWN arrow keys are pressed, activates current
         * selection if RETURN is pressed
         * 
         * @param e the keyboard event to process
         */
        @Override
        public void keyReleased(KeyEvent e) {
            switch (e.getKeyCode()) {
            case KeyEvent.VK_DOWN:
                moveSelectionDown();
                break;
            case KeyEvent.VK_UP:
                moveSelectionUp();
                break;
            case KeyEvent.VK_ENTER:
                if (isInButtonIndexRange(selectedItemIndex)) {
                    buttons.get(selectedItemIndex).activate();
                }
                break;
            }

            repaint();
        }

    }

    /**
     * Inner class to handle mouse events
     *
     * @author zsmb
     */
    private class ClickDetector extends MouseAdapter {

        /**
         * Unselects all Buttons when the mouse is released
         */
        @Override
        public void mouseReleased(MouseEvent e) {
            selectedItemIndex = -1;
        }

    }

    /**
     * Part of the interface the MenuPanel provides to its contained Buttons, allows
     * Buttons to signal being activated
     * 
     * @param s the received signal to process
     */
    @Override
    public void signal(MenuSignal s) {

        switch (s) {
        case Resume:
            gfa.signal(FrameSignal.Resume);
            break;
        case Play:
            gfa.signal(FrameSignal.Play);
            break;
        case Stats:
            gfa.signal(FrameSignal.Stats);
            break;
        case Exit:
            gfa.signal(FrameSignal.Exit);
            break;
        }
    }

    /**
     * Force a repaint of the MenuPanel
     */
    @Override
    public void requestRepaint() {
        repaint();
    }

}
