package zkp;

import java.io.Serializable;
import java.util.TreeMap;

import javax.swing.JFrame;

/**
 * This class provides an easy way to display different full-size panels inside a single
 * JFrame, ideal for game menus and such. Must be used with RepaintablePanels, which are
 * just JPanels with an extra function.
 *
 * @author zsmb
 */
public class PanelViewManager implements Serializable {

    private static final long serialVersionUID = 1041178567086476128L;

    /** TreeMap containing the panels, sorted by their indexes */
    private TreeMap<Integer, RepaintablePanel> panels 
        = new TreeMap<Integer, RepaintablePanel>();
    /** the JFrame to manage */
    private JFrame frame;
    /** reference to the panel currently in view */
    private RepaintablePanel panelInFrame = null;
    /** index of the panel currently in view */
    private int displayIndex = -1;

    /**
     * PanelViewManager ctor
     * 
     * @param frame the JFrame to display the panels in
     */
    public PanelViewManager(JFrame frame) {
        this.frame = frame;
    }

    /**
     * Check if there's a panel at a given index
     * 
     * @param i the index to check
     * @return true if there's already a panel at that index
     */
    public boolean exists(int i) {
        return panels.containsKey(i);
    }

    /**
     * Adds a panel to the given index
     * 
     * @param panel the panel to add
     * @param i the panel's index
     */
    public void add(RepaintablePanel panel, int i) {
        if (panel == null) {
            return;
        }

        panels.put(i, panel);

        if (displayIndex == i) {
            show(i);
        }
    }

    /**
     * Removes a panel from a given index
     * 
     * @param i the index of the panel to removes
     * @throws IllegalArgumentException thrown when index is invalid
     */
    public void remove(int i) throws IllegalArgumentException {
        if (!panels.containsKey(i)) {
            throw new IllegalArgumentException("No panel found at index " + i);
        }

        if (i != displayIndex) {
            panels.remove(i);
        }
    }

    /**
     * Shows the panel specified by the index inside the JFrame
     * 
     * @param i the index of the panel to display
     * @throws IllegalArgumentException thrown when index is invalid
     */
    public void show(int i) throws IllegalArgumentException {
        if (!panels.containsKey(i)) {
            throw new IllegalArgumentException("No panel found at index " + i);
        }

        if (panelInFrame != null) {
            frame.remove(panelInFrame);
            panelInFrame.setVisible(false);
        }

        displayIndex = i;
        panelInFrame = panels.get(i);
        panelInFrame.setVisible(true);
        frame.add(panelInFrame);
        frame.setVisible(true);

        panelInFrame.requestRepaint();
    }

    /**
     * Returns the panel at the given index
     * 
     * @param i the desired panel's index
     * @return the panel
     * @throws IllegalArgumentException thrown when index is invalid
     */
    public RepaintablePanel get(int i) throws IllegalArgumentException {
        if (!panels.containsKey(i)) {
            throw new IllegalArgumentException("No panel found at index " + i);
        } else {
            return panels.get(i);
        }
    }

}
