package zkp;

import java.io.InputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Random;
import java.util.Scanner;
import java.util.Set;
import java.util.TreeMap;

import zkp.GamePanel.ShipClass;

/**
 * A class that provides randomly selected words for a given ShipClass
 *
 * @author zsmb
 */
public class RandomWordProvider implements Serializable {

    private static final long serialVersionUID = -5404559260062141934L;

    /**
     * Counts of how many words starting with each key character are already in the game
     */
    private TreeMap<Character, Integer> charCounts = new TreeMap<Character, Integer>();

    /**
     * Stores all words read from file, first by ShipClass, secondly by key characters
     */
    private Map<ShipClass, TreeMap<Character, HashSet<String>>> wordStorage 
        = new HashMap<ShipClass, TreeMap<Character, HashSet<String>>>();

    /**
     * RandomWordProvider ctor
     */
    public RandomWordProvider() {
        for (char c = 'a'; c <= 'z'; c++) {
            charCounts.put(c, 0);
        }

        HashMap<ShipClass, InputStream> initMap = new HashMap<ShipClass, InputStream>();

        // Prepare the streams to read from
        initMap.put(ShipClass.Explorer, this.getClass().getResourceAsStream("/resources/ExplorerShip.txt"));
        initMap.put(ShipClass.Mothership, this.getClass().getResourceAsStream("/resources/MotherShip.txt"));
        initMap.put(ShipClass.Shuttle, this.getClass().getResourceAsStream("/resources/ShuttleShip.txt"));
        initMap.put(ShipClass.Simple, this.getClass().getResourceAsStream("/resources/SimpleShip.txt"));
            
        // Read each file
        for (Entry<ShipClass, InputStream> e : initMap.entrySet()) {
            // Create map for current ShipClass
            wordStorage.put(e.getKey(), new TreeMap<Character, HashSet<String>>());

            // Create submaps for all key characters
            for (char c = 'a'; c <= 'z'; c++) {
                wordStorage.get(e.getKey()).put(c, new HashSet<String>());
            }

            
                Scanner in = new Scanner(e.getValue());
                // Read file
                while (in.hasNextLine()) {
                    String temp = in.nextLine();
                    char keyChar = temp.charAt(0);
                    // Put each word into their appropriate word set
                    wordStorage.get(e.getKey()).get(keyChar).add(temp);
                }
                in.close();
            
        }

    }

    /**
     * Recalculate the contents of charCounts, based on the current EnemyShips on the map
     * 
     * @param ships a set of all active EnemyShips
     */
    private void reInitialize(Set<EnemyShip> ships) {
        for (char c = 'a'; c <= 'z'; c++) {
            charCounts.put(c, 0);
        }

        for (EnemyShip es : ships) {
            char keyChar = es.getKeyChar();
            charCounts.put(keyChar, charCounts.get(keyChar) + 1);
        }
    }

    /**
     * Gets the number of times the least frequent characters occured
     * 
     * @return occurances of the least frequent characters
     */
    private int getMinCount() {
        int minCount = Integer.MAX_VALUE;
        for (Entry<Character, Integer> e : charCounts.entrySet()) {
            if (e.getValue() < minCount && e.getKey() != 'x') {
                minCount = e.getValue();
            }
        }
        return minCount;
    }

    /**
     * Gets a list of the least frequent characters
     * 
     * @return the least frequent characters
     */
    private List<Character> getMinCountChars() {
        int minCount = getMinCount();

        ArrayList<Character> chars = new ArrayList<Character>();

        for (Map.Entry<Character, Integer> e : charCounts.entrySet()) {
            if (e.getValue() == minCount) {
                chars.add(e.getKey());
            }
        }

        if (chars.size() == 0) {
            for (char c = 'a'; c <= 'z'; c++) {
                chars.add(c);
            }
        }

        return chars;
    }

    /**
     * Randomizes an appropriate word for a given ShipClass, considering the other
     * EnemyShips already in the game
     * 
     * @param sc the ShipClass to get a word for
     * @param ships all active EnemyShips in the game
     * @return the chosen word
     */
    public String nextWord(ShipClass sc, Set<EnemyShip> ships) {
        // Refresh counts with new EnemyShip data
        this.reInitialize(ships);

        // Get the characters that occur the least amongst the active EnemyShips
        List<Character> minCountChars = getMinCountChars();

        Random r = new Random();

        // If we're dealing with a FighterShip, just return the one
        // of the minimum count characters as a String
        if (sc == ShipClass.Fighter) {
            return minCountChars.get(r.nextInt(minCountChars.size())).toString();
        }

        // A list of suitable words based on the minChars
        List<String> matchedWords = new ArrayList<String>();

        // Build list
        while (matchedWords.size() == 0) {
            Character keyChar = minCountChars.get(r.nextInt(minCountChars.size()));
            matchedWords.addAll(wordStorage.get(sc).get(keyChar));
        }

        // Return a random word from the list
        String word = matchedWords.get(r.nextInt(matchedWords.size()));

        return word;
    }

}
