package zkp;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import zkp.GamePanel.ShipClass;

/**
 * The storage class that can manage the statistics collected while playing the game
 *
 * @author zsmb
 */
public class StatCollector implements Serializable {

    private static final long serialVersionUID = -9134222123496832943L;

    /** The number of scores to store at a time */
    private static final int numOfStoredScores = 5;

    /** Accumulated score on the current level */
    private int currentScore = 0;

    /** STATS **/
    private int wordsTyped = 0; // Total number of words typed
    private int secondsPlayed = 0; // Total number of seconds played
    private List<Integer> scores; // Top scores achieved
    private Map<ShipClass, Integer> defeatedShips; // Count of defeated ships by
                                                   // type

    private int lastCommitedScore = 0; // The last achieved score that was saved

    /**
     * StatCollector ctor
     */
    public StatCollector() {
        // Initialize list with scores of all 0
        scores = new ArrayList<Integer>();
        for (int i = 0; i < numOfStoredScores; i++) {
            scores.add(0);
        }

        // Initialize map with 0 defeated ships of each type
        defeatedShips = new HashMap<ShipClass, Integer>();
        for (ShipClass sc : ShipClass.values()) {
            defeatedShips.put(sc, 0);
        }
    }

    /**
     * Registers an EnemyShip of a certain class being killed
     * 
     * @param sc the killed EnemyShip's type
     */
    public void addDefeatedShip(ShipClass sc) {
        int prev = defeatedShips.get(sc);
        defeatedShips.put(sc, prev + 1);

        switch (sc) {
        case Fighter:
            currentScore += 1;
            break;
        case Shuttle:
            currentScore += 5;
            break;
        case Simple:
            currentScore += 10;
            break;
        case Explorer:
            currentScore += 25;
            break;
        case Mothership:
            currentScore += 50;
            break;
        }
    }

    /**
     * Adds a score to the stored ones, if it's deemed worthy
     * 
     * @param score the new score to add
     */
    private void addScore(int score) {
        if (score < 0) {
            return;
        }

        scores.add(score);
        Collections.sort(scores, Collections.reverseOrder());
        scores.remove(numOfStoredScores);
    }

    /**
     * Adds to the total word and second counts
     * 
     * @param words number of words typed
     * @param seconds number of seconds played
     */
    public void addWordPerMinStat(int words, int seconds) {
        wordsTyped += words;
        secondsPlayed += seconds;
    }

    /**
     * Commits accumulated score on a level into the list of archived scores, resets
     * accumulated score
     */
    public void commitScore() {
        addScore(currentScore);
        lastCommitedScore = currentScore;
        currentScore = 0;
    }

    /**
     * Getter function
     * 
     * @return the last commited score
     */
    public int getLastCommitedScore() {
        return lastCommitedScore;
    }

    /**
     * Getter function for number of EnemyShips of a given type killed
     * 
     * @param sc the type of EnemyShip
     * @return number of the given type killed
     */
    public int getDefeatedShips(ShipClass sc) {
        return defeatedShips.get(sc);
    }

    /**
     * Getter function for the percentage of EnemyShips killed the given type accounts for
     * 
     * @param sc the type to get the percentage for
     * @return ratio of the given type killed (0.0-1.0)
     */
    public double getPercetangeOfDefeatedShips(ShipClass sc) {
        return Math.min((double) defeatedShips.get(sc) / wordsTyped, 1);
    }

    /**
     * Getter function
     * 
     * @return list of stored scores
     */
    public List<Integer> getScores() {
        ArrayList<Integer> temp = new ArrayList<Integer>(scores);
        return temp;
    }
    

    /**
     * Getter function
     * 
     * @return total number of seconds played
     */
    public int getTotalSeconds() {
        return secondsPlayed;
    }
    

    /**
     * Getter function
     * 
     * @return total number of words typed
     */
    public int getWords() {
        return wordsTyped;
    }
    
    /**
     * Getter function
     * 
     * @return average number of words typed per minute
     */
    public double getWordsPerMinute() {
        if (secondsPlayed == 0) {
            return 0;
        }

        return (double) wordsTyped / (secondsPlayed / 60.0);
    }

}
