package zkp;

import java.awt.Graphics;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import zkp.GameFrameAccessProvider.FrameSignal;
import zkp.GamePanel.ShipClass;

/**
 * A menu screen that displays the collected gameplay statistics and high scores
 *
 * @author zsmb
 */
public class StatPanel extends RepaintablePanel {

    private static final long serialVersionUID = -2118042026226995531L;

    private static final int boxColour = 0x666666FF; // background colour of each
                                                     // displayed row
    private static final int dataColour = 0x66DDDDFF; // text colour
    private static final int highlightColour = 0x99EE99FF; // highlighted text colour

    private int width, height; // width and height in pixels
    private StatCollector sc; // StatCollector to read stats from

    /**
     * StatPanel ctor
     * 
     * @param gfa the containing GameFrame's interface for communication
     * @param width width in pixels
     * @param height height in pixels
     * @param sc the source for displayed stats
     */
    public StatPanel(GameFrameAccessProvider gfa, int width, int height, StatCollector sc) {
        this.width = width;
        this.height = height;
        this.sc = sc;

        this.addKeyListener(new KeyAdapter() {
            
            @Override
            public void keyReleased(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ESCAPE
                        || e.getKeyCode() == KeyEvent.VK_BACK_SPACE
                        || e.getKeyCode() == KeyEvent.VK_SPACE
                        || e.getKeyCode() == KeyEvent.VK_ENTER) {
                    gfa.signal(FrameSignal.Menu);
                }
            }
            
        });
    }

    /**
     * Display stats on the screen
     * 
     * @param g the Graphics object to use for painting
     */
    @Override
    public void paint(Graphics g) {

        GFX.rect(g, 0, 0, width, height, 0xAAAAAAFF);

        this.y = 20;

        advanceLine();
        displayHeaderText(g, "Scores", dataColour);
        for (int i = 0; i < sc.getScores().size(); i++) {
            if (gameEnded && sc.getScores().get(i) == sc.getLastCommitedScore()) {
                displayTextWithValue(g, "#" + i, sc.getScores().get(i), highlightColour);
                gameEnded = false;
            } else {
                displayTextWithValue(g, "#" + i, sc.getScores().get(i), dataColour);
            }
        }

        advanceLine();
        displayHeaderText(g, "General stats", dataColour);
        displayTextWithValue(g, "WPM", sc.getWordsPerMinute(), dataColour);
        displayTextWithValue(g, "Total words typed", sc.getWords(), dataColour);
        displayTextWithValue(g, "Total minutes played", sc.getTotalSeconds()/60, dataColour);

        advanceLine();
        displayHeaderText(g, "Kills by ship type", dataColour);
        for (ShipClass s : ShipClass.values()) {
            displayTextWithValueAndPercentage(g, s.toString(), sc.getDefeatedShips(s),
                    sc.getPercetangeOfDefeatedShips(s), dataColour);
        }
        
        // fixed text, don't modify
        advanceLine();
        GFX.aaText(g, width/2, y, "Press SPACE to return to menu", boxColour, 0, 20);
    }

    /**
     * Getter function
     * 
     * @return the horizontal middle point of the screen in pixels
     */
    private int getCenter() {
        return this.width / 2;
    }

    /**
     * The next height to put a stat entry<br>
     * The functions in this class can me used modularly after each other as necessary,
     * heights will be automatically managed
     */
    private int y;

    /**
     * Creates an empty line
     */
    private void advanceLine() {
        advanceLine(20);
    }

    /**
     * Advance the current height by a specific amount (based on font size of the last
     * displayed entry
     * 
     * @param amount
     */
    private void advanceLine(int amount) {
        y += amount * 1.5;
    }

    private final int standardSize = 20; // font size for stat entries
    private final int headerSize = 26; // font size for headers

    private final int largeMargin = 40; // outer margins for the displayed boxes
    private final int middleMargin = 20; // inside padding for the displayed
                                         // boxes
    private final int smallMargin = 8; // inside padding for header text boxes

    //
    // <--> largeMargin <-->
    // <--> smallMargin <-->
    // +----------------------------+
    // | Header |
    // | |
    // | Entry 100 |
    // +----------------------------+
    // <----> middleMargin <---->
    //

    /**
     * Gets half the width of a displayed entry box
     * 
     * @return the half width
     */
    private int getHalfWidth() {
        return (width - 2 * largeMargin) / 2;
    }

    /**
     * Displays a stat entry with a name and a floating point value
     * 
     * @param g the Graphics object to use for painting
     * @param text the entry's label
     * @param value the entry's value
     * @param colour font colour in 0xRRGGBBAA format
     */
    private void displayTextWithValue(Graphics g, String text, double value, int colour) {
        int x = getCenter();
        int hw = getHalfWidth();
        GFX.rect(g, x - hw, y - standardSize * 1.2, hw * 2, standardSize * 1.7, boxColour);
        GFX.aaText(g, x - hw + middleMargin, y, text, colour, -1, standardSize);
        GFX.aaText(g, x + hw - middleMargin, y, String.format("%.3f", value), colour, 1, standardSize);
        advanceLine(standardSize);
    }

    /**
     * Displays a stat entry with a name and an integer value
     * 
     * @param g the Graphics object to use for painting
     * @param text the entry's label
     * @param value the entry's value
     * @param colour font colour in 0xRRGGBBAA format
     */
    private void displayTextWithValue(Graphics g, String text, int value, int colour) {
        int x = getCenter();
        int hw = getHalfWidth();
        GFX.rect(g, x - hw, y - standardSize * 1.2, hw * 2, standardSize * 1.7, boxColour);
        GFX.aaText(g, x - hw + middleMargin, y, text, colour, -1, standardSize);
        GFX.aaText(g, x + hw - middleMargin, y, String.format("%d", value), colour, 1, standardSize);
        advanceLine(standardSize);
    }

    /**
     * Displays a stat entry with a name and an integer value and a background bar filled
     * up to the given percentage
     * 
     * @param g the Graphics object to use for painting
     * @param text the entry's label
     * @param value the entry's value
     * @param percentage the percentage that this stat entry represents
     * @param colour font colour in 0xRRGGBBAA format
     */
    private void displayTextWithValueAndPercentage(Graphics g, String text, int value, double percentage, int colour) {
        int x = getCenter();
        int hw = getHalfWidth();
        GFX.rect(g, x - hw, y - standardSize * 1.2, hw * 2, standardSize * 1.7, boxColour);
        GFX.rect(g, x - hw, y - standardSize * 1.2, hw * 2 * percentage, standardSize * 1.7, 0x66AAAA99);
        GFX.aaText(g, x - hw + middleMargin, y, text, colour, -1, standardSize);
        GFX.aaText(g, x + hw - middleMargin, y, String.format("%d", value), colour, 1, standardSize);
        advanceLine(standardSize);
    }

    /**
     * Displays a header
     * 
     * @param g the Graphics object to use for painting
     * @param text the header's text
     * @param colour font colour in 0xRRGGBBAA format
     */
    private void displayHeaderText(Graphics g, String text, int colour) {
        int x = getCenter();
        int hw = getHalfWidth();
        GFX.rect(g, x - hw, y - headerSize * 1.2, hw * 2, headerSize * 1.7, boxColour);
        GFX.aaText(g, x - hw + smallMargin, y, text, colour, -1, headerSize);
        advanceLine(headerSize);
    }

    /**
     * Setting this flag lets the StatPanel know to display itself slightly differently
     * than when accessed from the main menu screen
     */
    private boolean gameEnded = false;

    /**
     * Signals the StatPanel about a game ending
     */
    public void notifyGameEnded() {
        this.gameEnded = true;
    }

    /**
     * Force a repaint of the StatPanel
     */
    @Override
    public void requestRepaint() {
        repaint();
    }

    /**
     * Required for listeners to keep working while switching panels
     */
    public void addNotify() {
        super.addNotify();
        requestFocus();
    }

}
